<?php

use CRM_Bij1migratie_Mapping_AirtableMapping as AirtableMapping;
use CRM_Bij1migratie_Mapping_BetalingenMapping as BetalingenMapping;
use CRM_Bij1migratie_Mapping_ChargebacksMapping as ChargebacksMapping;
use CRM_Bij1migratie_Mapping_PersonenMapping as PersonenMapping;
use CRM_Bij1migratie_ExtensionUtil as E;

/**
 * Class voor BIJ1 migratie CSV import
 *
 * @author Erik Hommel (CiviCooP) <erik.hommel@civicoop.org>
 * @date 9 dec 2021
 * @license AGPL-3.0
 */
class CRM_Bij1migratie_CsvFile {

  private $_separator = NULL;
  private $_fileName = NULL;
  private $_usesHeaders = NULL;
  private $_rowNumber = NULL;
  private $_headers = [];
  private $_migratieType = NULL;
  public $csv = NULL;

  /**
   * Method to initialize
   *
   * @param string $fileName
   * @param string $migratieType
   * @param bool $usesHeaders
   * @param string $separator
   */
  public function initialize(string $fileName, string $migratieType, bool $usesHeaders = FALSE, string $separator = "~") {
    $this->_fileName = $fileName;
    $this->_separator = $separator;
    $this->_usesHeaders = $usesHeaders;
    $this->_rowNumber = 0;
    $this->_migratieType = $migratieType;
  }

  /**
   * Method to get the headers (and numbers if none found)
   *
   * @throws Exception if file can not be opened
   */
  public function getHeaders() {
    $headers = [];
    $i = 0;
    $this->open();
    if ($this->_usesHeaders) {
      $data = fgetcsv($this->csv, 0, $this->_separator);
      foreach ($data as $key => $value) {
        if ($value) {
          $headers[] = $value;
          $i++;
        }
        else {
          $i++;
          $headers[] = "csv column " . $i;
        }
      }
    }
    else {
      $data = fgetcsv($this->csv, 0, $this->_separator);
      $count = count($data);
      while ($i < $count) {
        $i++;
        $headers[] = "csv column " . $i;
      }
    }
    $this->close();
    return $headers;
  }

  /**
   * Method to open the file for read
   *
   * @throws Exception when file can not be opened
   * @return bool
   */
  public function open() {
    $this->csv = fopen($this->_fileName, 'r');
    if ($this->csv) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Method to read the next line and return an array with data
   *
   * @param string $tableName
   * @return array|bool
   */
  public function readNext(string $tableName) {
    $this->_rowNumber++;
    // if row 1 complete mapping and if headers, skip
    if ($this->_rowNumber == 1) {
      $data = fgetcsv($this->csv, 0, $this->_separator);
      $this->setHeaders($data);
      if (!$this->_usesHeaders) {
        rewind($this->csv);
      }
    }
    $data = fgetcsv($this->csv, 0, $this->_separator);
    if ($tableName && $data) {
      $this->sanitizeData($tableName, $data);
    }
    if ($data) {
      return $data;
    }
    return FALSE;
  }

  /**
   * Sanitize row: empty data if it contains invalid content.
   * Iterates over any column of intermediary table $tableName
   * to transform source columns into.
   *
   * @param string $tableName
   * @param array $data
   * @throws
   */
  private function sanitizeData(string $tableName, array &$data) {
    if (!empty($data)) {
      $table = new CRM_Bij1migratie_ImportTable();
      $columns = $table->getColumnNames($tableName);
      foreach ($columns as $key => $name) {
        switch ($this->_migratieType) {
          case "A":
            $this->sanitizeAirtable($name, $data);
            break;
          case "B":
            $this->sanitizeBetalingen($name, $data);
            break;
          case "C":
            $this->sanitizeChargebacks($name, $data);
            break;
          case "P":
            $this->sanitizePersonen($name, $data);
            break;
        }
      }
      foreach ($data as $key => $value) {
        if (is_numeric($key)) {
          unset($data[$key]);
        }
      }
    }
  }

  /**
   * Method om airtable data op te schonen voor form processor verwerking
   *
   * @param string $name
   * @param array $data
   * @throws Exception
   */
  private function sanitizeAirtable(string $name, array &$data) {
    $mapping = new AirtableMapping($data);
    $value = $mapping->get($name);

    switch ($name) {
      case AirtableMapping::FIELD_VOORNAAM:
      case AirtableMapping::FIELD_ACHTERNAAM:
      case AirtableMapping::FIELD_EMAIL:
      case AirtableMapping::FIELD_STRAAT_HUISNUMMER:
      case AirtableMapping::FIELD_POSTCODE:
      case AirtableMapping::FIELD_PLAATS:
      case AirtableMapping::FIELD_NOTITIE:
      case AirtableMapping::FIELD_TELEFOON:
      case AirtableMapping::FIELD_IBAN:
      case AirtableMapping::FIELD_MOLLIE_CUSTOMER_ID:
      case AirtableMapping::FIELD_PERIODE:
      case AirtableMapping::FIELD_TERMIJNBEDRAG:
      case AirtableMapping::FIELD_TRIODOS:
      // case AirtableMapping::FIELD_SOCIAAL_TARIEF:
      //   $data[$name] = $value;
      //   break;
      case AirtableMapping::FIELD_LIDSTATUS:
        $data[$name] = strtolower($value);
        break;
      case AirtableMapping::FIELD_LAND:
        $landId = Civi::service('bij1Migratie')->haalAirtableLand(trim(strtolower($value)));
        if ($landId) {
          $data[$name] = (int) $landId;
        }
        else {
          $data[$name] = 1152;
        }
        break;
      case AirtableMapping::FIELD_INSCHRIJVINGSDATUM:
        if (!empty($value)) {
          $value = str_replace("/", "-", $value);
          $inschrijvingsdatum = new DateTime($value);
          $data[$name] = $inschrijvingsdatum->format("d-m-Y");
        }
        break;
      case AirtableMapping::FIELD_LIDNUMMER:
        if (is_numeric($value)) {
          $data[$name] = $value;
        }
        break;
    }
  }

  /**
   * Method om data voor betalingen samen te stellen
   *
   * @param string $name
   * @param string $key
   * @param array $data
   * @throws
   */
  private function sanitizeBetalingen(string $name, array &$data) {
    $mapping = new BetalingenMapping($data);
    $value = $mapping->get($name);

    switch ($name) {
      case BetalingenMapping::FIELD_TRANSACTIE_ID:
      case BetalingenMapping::FIELD_STATUS:
      case BetalingenMapping::FIELD_MOLLIE_CUSTOMER_ID:
      case BetalingenMapping::FIELD_MANDAAT:
      case BetalingenMapping::FIELD_BEDRAG:
      case BetalingenMapping::FIELD_MUNTEENHEID:
      case BetalingenMapping::FIELD_KLANT:
      case BetalingenMapping::FIELD_EMAIL:
        $data[$name] = $value;
        break;
      case BetalingenMapping::FIELD_AANMAAKDATUM:
        if (!empty($value)) {
          $donatieDatum = new DateTime($value);
          $data[$name] = $donatieDatum->format('d-m-Y');
        }
        break;
      case BetalingenMapping::FIELD_BETALINGSDATUM:
        if (!empty($value)) {
          $betalingsDatum = new DateTime($value);
          $data[$name] = $betalingsDatum->format('d-m-Y');
        }
        break;
      case BetalingenMapping::FIELD_OMSCRHIJVING:
        if (!empty($value)) {
          $data[$name] = $value;
        }
        else {
          $data[$name] = "Migratie CiviCRM";
        }
        break;
      case BetalingenMapping::FIELD_BETALINGSMETHODE:
        if (!empty($value)) {
          $data[$name] = $value;
        }
        break;
      case BetalingenMapping::FIELD_BETALINGSINSTRUMENT:
        if (!empty($value)) {
          $data[$name] = Civi::service('bij1Migratie')->haalBetalingsinstrument($value);
        }
        break;
    }
  }

  /**
   * Method om data voor chargebacks samen te stellen
   *
   * @param string $name
   * @param string $key
   * @param array $data
   * @throws
   */
  private function sanitizeChargebacks(string $name, array &$data) {
    $mapping = new ChargebacksMapping($data);
    $value = $mapping->get($name);

    switch ($name) {
      case ChargebacksMapping::FIELD_CHARGEBACK_ID:
      case ChargebacksMapping::FIELD_TRANSACTIE_ID:
      case ChargebacksMapping::FIELD_REDEN_CODE:
      case ChargebacksMapping::FIELD_REDEN_OMSCHRIJVING:
      case ChargebacksMapping::FIELD_BEDRAG:
      case ChargebacksMapping::FIELD_MOLLIE_CUSTOMER_ID:
        $data[$name] = $value;
        break;
      case ChargebacksMapping::FIELD_CHARGEBACK_DATUM:
        if (!empty($value)) {
          $donatieDatum = new DateTime($value);
          $data[$name] = $donatieDatum->format('d-m-Y');
        }
        break;
      case ChargebacksMapping::FIELD_CHARGEBACK_BEDRAG:
        $data[$name] = abs($value);
        break;
    }
  }

  /**
   * Method om data voor personen samen te stellen.
   * Behandelde kolommen zijn voornaam, achternaam, email, mollie_customer_id.
   *
   * @param string $name
   * @param array $data
   * @throws Exception
   */
  private function sanitizePersonen(string $name, array &$data) {
    $mapping = new PersonenMapping($data);

    switch ($name) {
      case PersonenMapping::FIELD_MOLLIE_CUSTOMER_ID:
      case PersonenMapping::FIELD_EMAIL:
        $data[$name] = $mapping->get($name);
        break;
      // Gezien naam samen zit in de bron data moeten we deze hier splitsen.
      case PersonenMapping::FIELD_VOORNAAM:
        $value = $mapping->get(PersonenMapping::FIELD_NAME);
        $naamDelen = explode(" ", $value, 2);
        if (count($naamDelen) > 1) {
          $data[PersonenMapping::FIELD_VOORNAAM] = trim($naamDelen[0]);
          $data[PersonenMapping::FIELD_ACHTERNAAM] = trim($naamDelen[1]);
        }
        else {
          if (!empty($naamDelen[0])) {
            $data[PersonenMapping::FIELD_ACHTERNAAM] = trim($naamDelen[0]);
          }
          else {
            $data[PersonenMapping::FIELD_ACHTERNAAM] = "Onbekend " . rand(0,9999);
          }
          $data[PersonenMapping::FIELD_VOORNAAM] = "Onbekend " . rand(0,9999);
        }
        break;
      // case PersonenMapping::FIELD_ACHTERNAAM:
    }
  }

  /**
   * Method to close the file
   */
  public function close() {
    fclose($this->csv);
  }

  /**
   * Method to set the headers of the csv file
   *
   * @param $headers
   */
  private function setHeaders($headers) {
    $this->_headers = $headers;
  }

}
